@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the qmake application of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "msbuild_objectmodel.h"
 
@@ -34,7 +9,8 @@
 #include <qscopedpointer.h>
 #include <qstringlist.h>
 #include <qfileinfo.h>
-#include <qversionnumber.h>
+#include <qregularexpression.h>
+#include <qvarlengtharray.h>
 
 QT_BEGIN_NAMESPACE
 
@@ -143,6 +119,8 @@ const char _InterfaceIdentifierFileName[]       = "InterfaceIdentifierFileName";
 const char _IntermediateDirectory[]             = "IntermediateDirectory";
 const char _KeyContainer[]                      = "KeyContainer";
 const char _KeyFile[]                           = "KeyFile";
+const char _LanguageStandard[]                  = "LanguageStandard";
+const char _LanguageStandard_C[]                = "LanguageStandard_C";
 const char _LargeAddressAware[]                 = "LargeAddressAware";
 const char _LinkDLL[]                           = "LinkDLL";
 const char _LinkErrorReporting[]                = "LinkErrorReporting";
@@ -299,14 +277,17 @@ inline XmlOutput::xml_output valueTagT( const triState v)
     return valueTag(v == _True ? "true" : "false");
 }
 
-static QString vcxCommandSeparator()
+static QString commandLinesForOutput(QStringList commands)
 {
     // MSBuild puts the contents of the custom commands into a batch file and calls it.
     // As we want every sub-command to be error-checked (as is done by makefile-based
     // backends), we insert the checks ourselves, using the undocumented jump target.
-    static QString cmdSep =
-    QLatin1String("&#x000D;&#x000A;if errorlevel 1 goto VCEnd&#x000D;&#x000A;");
-    return cmdSep;
+    static QString errchk = QStringLiteral("if errorlevel 1 goto VCEnd");
+    for (int i = commands.count() - 2; i >= 0; --i) {
+        if (!commands.at(i).startsWith("rem", Qt::CaseInsensitive))
+            commands.insert(i + 1, errchk);
+    }
+    return commands.join("\r\n");
 }
 
 static QString unquote(const QString &value)
@@ -608,14 +589,12 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
         << tag("ItemGroup")
         << attrTag("Label", "ProjectConfigurations");
 
-    bool isWinRT = false;
     for (int i = 0; i < tool.SingleProjects.count(); ++i) {
         xml << tag("ProjectConfiguration")
             << attrTag("Include" , tool.SingleProjects.at(i).Configuration.Name)
             << tagValue("Configuration", tool.SingleProjects.at(i).Configuration.ConfigurationName)
             << tagValue("Platform", tool.SingleProjects.at(i).PlatformName)
             << closetag();
-        isWinRT = isWinRT || tool.SingleProjects.at(i).Configuration.WinRT;
     }
 
     xml << closetag()
@@ -625,31 +604,10 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
         << tagValue("RootNamespace", tool.Name)
         << tagValue("Keyword", tool.Keyword);
 
-    QString windowsTargetPlatformVersion;
-    if (isWinRT) {
-        xml << tagValue("MinimumVisualStudioVersion", tool.Version)
-            << tagValue("DefaultLanguage", "en")
-            << tagValue("AppContainerApplication", "true")
-            << tagValue("ApplicationType", "Windows Store")
-            << tagValue("ApplicationTypeRevision", tool.SdkVersion);
-        if (tool.SdkVersion == "10.0")
-            windowsTargetPlatformVersion = qgetenv("UCRTVERSION");
-    } else {
-        QByteArray winSDKVersionStr = qgetenv("WindowsSDKVersion").trimmed();
-
-        // This environment variable might end with a backslash due to a VS bug.
-        if (winSDKVersionStr.endsWith('\\'))
-            winSDKVersionStr.chop(1);
-
-        QVersionNumber winSDKVersion = QVersionNumber::fromString(
-                    QString::fromLocal8Bit(winSDKVersionStr));
-        if (!winSDKVersion.isNull())
-            windowsTargetPlatformVersion = winSDKVersionStr;
-    }
-    if (!windowsTargetPlatformVersion.isEmpty()) {
-        xml << tagValue("WindowsTargetPlatformVersion", windowsTargetPlatformVersion)
-            << tagValue("WindowsTargetPlatformMinVersion", windowsTargetPlatformVersion);
-    }
+    if (!tool.WindowsTargetPlatformVersion.isEmpty())
+        xml << tagValue("WindowsTargetPlatformVersion", tool.WindowsTargetPlatformVersion);
+    if (!tool.WindowsTargetPlatformMinVersion.isEmpty())
+        xml << tagValue("WindowsTargetPlatformMinVersion", tool.WindowsTargetPlatformMinVersion);
 
     xml << closetag();
 
@@ -835,40 +793,6 @@ void VCXProjectWriter::write(XmlOutput &xml, VCProject &tool)
     }
     outputFilter(tool, xml, xmlFilter, "Root Files");
 
-    // App manifest
-    if (isWinRT) {
-        const QString manifest = QStringLiteral("Package.appxmanifest");
-
-        // Find all icons referenced in the manifest
-        QSet<QString> icons;
-        QFile manifestFile(Option::output_dir + QLatin1Char('/') + manifest);
-        if (manifestFile.open(QFile::ReadOnly)) {
-            const QString contents = manifestFile.readAll();
-            QRegExp regexp("[\\\\/a-zA-Z0-9_\\-\\!]*\\.(png|jpg|jpeg)");
-            int pos = 0;
-            while (pos > -1) {
-                pos = regexp.indexIn(contents, pos);
-                if (pos >= 0) {
-                    const QString match = regexp.cap(0);
-                    icons.insert(match);
-                    pos += match.length();
-                }
-            }
-        }
-
-        // Write out manifest + icons as content items
-        xml << tag(_ItemGroup)
-            << tag("AppxManifest")
-            << attrTag("Include", manifest)
-            << closetag();
-        for (const QString &icon : qAsConst(icons)) {
-            xml << tag("Image")
-                << attrTag("Include", icon)
-                << closetag();
-        }
-        xml << closetag();
-    }
-
     xml << import("Project", "$(VCTargetsPath)\\Microsoft.Cpp.targets")
         << tag("ImportGroup")
         << attrTag("Label", "ExtensionTargets")
@@ -1140,6 +1064,8 @@ static inline QString toString(optLinkTimeCodeGenType option)
         break;
     case optLTCGEnabled:
         return "UseLinkTimeCodeGeneration";
+    case optLTCGIncremental:
+        return "UseFastLinkTimeCodeGeneration";
     case optLTCGInstrument:
         return "PGInstrument";
     case optLTCGOptimize:
@@ -1173,6 +1099,8 @@ static inline QString toString(triState genDebugInfo, linkerDebugOption option)
     case _True:
         if (option == linkerDebugOptionFastLink)
             return "DebugFastLink";
+        else if (option == linkerDebugOptionFull)
+            return "DebugFull";
         return "true";
     }
     return QString();
@@ -1207,7 +1135,7 @@ static inline QString toString(midlCharOption option)
 static inline QString toString(midlErrorCheckOption option)
 {
     switch (option) {
-    case midlAlignNotSet:
+    case midlEnableCustom:
         break;
     case midlDisableAll:
         return "None";
@@ -1492,6 +1420,8 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCCLCompilerTool &tool)
             << attrTagT(_IntrinsicFunctions, tool.EnableIntrinsicFunctions)
             << attrTagT(_MinimalRebuild, tool.MinimalRebuild)
             << attrTagT(_MultiProcessorCompilation, tool.MultiProcessorCompilation)
+            << attrTagS(_LanguageStandard, tool.LanguageStandard)
+            << attrTagS(_LanguageStandard_C, tool.LanguageStandard_C)
             << attrTagS(_ObjectFileName, tool.ObjectFile)
             << attrTagT(_OmitDefaultLibName, tool.OmitDefaultLibName)
             << attrTagT(_OmitFramePointers, tool.OmitFramePointers)
@@ -1670,7 +1600,7 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCCustomBuildTool &tool)
     {
         xml << tag("Command")
             << attrTag("Condition", condition)
-            << valueTag(tool.CommandLine.join(vcxCommandSeparator()));
+            << valueTag(commandLinesForOutput(tool.CommandLine));
     }
 
     if ( !tool.Description.isEmpty() )
@@ -1724,7 +1654,7 @@ void VCXProjectWriter::write(XmlOutput &xml, const VCEventTool &tool)
 {
     xml
         << tag(tool.EventName)
-            << tag(_Command) << valueTag(tool.CommandLine.join(vcxCommandSeparator()))
+            << tag(_Command) << valueTag(commandLinesForOutput(tool.CommandLine))
             << tag(_Message) << valueTag(tool.Description)
         << closetag(tool.EventName);
 }
