@@ -1,30 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the qmake application of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include "qmakeevaluator.h"
 
@@ -38,16 +13,15 @@
 #include <qdir.h>
 #include <qfile.h>
 #include <qfileinfo.h>
+#include <qjsonarray.h>
+#include <qjsondocument.h>
+#include <qjsonobject.h>
 #include <qlist.h>
-#include <qregexp.h>
+#include <qregularexpression.h>
 #include <qset.h>
 #include <qstringlist.h>
 #include <qtextstream.h>
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
-# include <qjsondocument.h>
-# include <qjsonobject.h>
-# include <qjsonarray.h>
-#endif
+
 #ifdef PROEVALUATOR_THREAD_SAFE
 # include <qthreadpool.h>
 #endif
@@ -67,7 +41,7 @@
 #include <sys/stat.h>
 #include <sys/utsname.h>
 #else
-#include <windows.h>
+#include <qt_windows.h>
 #endif
 #include <stdio.h>
 #include <stdlib.h>
@@ -222,9 +196,7 @@ void QMakeEvaluator::initFunctionStatics()
         { "infile", T_INFILE, 2, 3, "file, var, [values]" },
         { "count", T_COUNT, 2, 3, "var, count, [op=operator]" },
         { "isEmpty", T_ISEMPTY, 1, 1, "var" },
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
         { "parseJson", T_PARSE_JSON, 2, 2, "var, into" },
-#endif
         { "load", T_LOAD, 1, 2, "feature, [ignore_errors=false]" },
         { "include", T_INCLUDE, 1, 3, "file, [into, [silent]]" },
         { "debug", T_DEBUG, 2, 2, "level, message" },
@@ -362,7 +334,6 @@ QMakeEvaluator::quoteValue(const ProString &val)
     return ret;
 }
 
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
 static void addJsonValue(const QJsonValue &value, const QString &keyPrefix, ProValueMap *map);
 
 static void insertJsonKeyValue(const QString &key, const QStringList &values, ProValueMap *map)
@@ -474,7 +445,6 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::parseJsonInto(const QByteArray &json
 
     return QMakeEvaluator::ReturnTrue;
 }
-#endif
 
 QMakeEvaluator::VisitReturn
 QMakeEvaluator::writeFile(const QString &ctx, const QString &fn, QIODevice::OpenMode mode,
@@ -639,7 +609,11 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         if (!var.isEmpty()) {
             const auto strings = values(map(var));
             if (regexp) {
-                QRegExp sepRx(sep);
+                QRegularExpression sepRx(sep, QRegularExpression::DotMatchesEverythingOption);
+                if (!sepRx.isValid()) {
+                    evalError(fL1S("section(): Encountered invalid regular expression '%1'.").arg(sep));
+                    goto allfail;
+                }
                 for (const ProString &str : strings) {
                     ProStringRwUser u1(str, m_tmp[m_toggle ^= 1]);
                     ret << u1.extract(u1.str().section(sepRx, beg, end));
@@ -669,7 +643,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         bool leftalign = false;
         enum { DefaultSign, PadSign, AlwaysSign } sign = DefaultSign;
         if (args.count() >= 2) {
-            const auto opts = split_value_list(args.at(1).toQStringRef());
+            const auto opts = split_value_list(args.at(1).toQStringView());
             for (const ProString &opt : opts) {
                 if (opt.startsWith(QLatin1String("ibase="))) {
                     ibase = opt.mid(6).toInt();
@@ -772,7 +746,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         const auto vars = values(map(args.at(0)));
         for (const ProString &var : vars) {
             // FIXME: this is inconsistent with the "there are no empty strings" dogma.
-            const auto splits = var.toQStringRef().split(sep, QString::KeepEmptyParts);
+            const auto splits = var.toQStringView().split(sep, Qt::KeepEmptyParts);
             for (const auto &splt : splits)
                 ret << ProString(splt).setSource(var);
         }
@@ -862,7 +836,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
                         ret += ProString(stream.readLine());
                     } else {
                         const QString &line = stream.readLine();
-                        ret += split_value_list(QStringRef(&line).trimmed());
+                        ret += split_value_list(QStringView(line).trimmed());
                         if (!singleLine)
                             ret += ProString("\n");
                     }
@@ -882,20 +856,23 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         ret += values(map(args.at(0)));
         break;
     case E_LIST: {
-        QString tmp;
-        tmp.sprintf(".QMAKE_INTERNAL_TMP_variableName_%d", m_listCount++);
+        QString tmp(QString::asprintf(".QMAKE_INTERNAL_TMP_variableName_%d", m_listCount++));
         ret = ProStringList(ProString(tmp));
         ProStringList lst;
         for (const ProString &arg : args)
-            lst += split_value_list(arg.toQStringRef(), arg.sourceFile()); // Relies on deep copy
+            lst += split_value_list(arg.toQStringView(), arg.sourceFile()); // Relies on deep copy
         m_valuemapStack.top()[ret.at(0).toKey()] = lst;
         break; }
     case E_FIND: {
-        QRegExp regx(args.at(1).toQString());
+        QRegularExpression regx(args.at(1).toQString(), QRegularExpression::DotMatchesEverythingOption);
+        if (!regx.isValid()) {
+            evalError(fL1S("find(): Encountered invalid regular expression '%1'.").arg(regx.pattern()));
+            goto allfail;
+        }
         const auto vals = values(map(args.at(0)));
         for (const ProString &val : vals) {
             ProStringRoUser u1(val, m_tmp[m_toggle ^= 1]);
-            if (regx.indexIn(u1.str()) != -1)
+            if (u1.str().contains(regx))
                 ret += val;
         }
         break;
@@ -932,7 +909,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
                 output.replace(QLatin1Char('\t'), QLatin1Char(' '));
                 if (singleLine)
                     output.replace(QLatin1Char('\n'), QLatin1Char(' '));
-                ret += split_value_list(QStringRef(&output));
+                ret += split_value_list(QStringView(output));
             }
         }
         break;
@@ -991,7 +968,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
     case E_RE_ESCAPE:
         for (int i = 0; i < args.size(); ++i) {
             ProStringRwUser u1(args.at(i), m_tmp1);
-            ret << u1.extract(QRegExp::escape(u1.str()));
+            ret << u1.extract(QRegularExpression::escape(u1.str()));
         }
         break;
     case E_VAL_ESCAPE: {
@@ -1039,12 +1016,16 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
             dirs.append(QString());
         }
 
-        r.detach(); // Keep m_tmp out of QRegExp's cache
-        QRegExp regex(r, Qt::CaseSensitive, QRegExp::Wildcard);
+        QString pattern = QRegularExpression::wildcardToRegularExpression(r);
+        QRegularExpression regex(pattern, QRegularExpression::DotMatchesEverythingOption);
+        if (!regex.isValid()) {
+            evalError(fL1S("section(): Encountered invalid wildcard expression '%1'.").arg(pattern));
+            goto allfail;
+        }
         for (int d = 0; d < dirs.count(); d++) {
             QString dir = dirs[d];
             QDir qdir(pfx + dir);
-            for (int i = 0; i < (int)qdir.count(); ++i) {
+            for (int i = 0, count = int(qdir.count()); i < count; ++i) {
                 if (qdir[i] == statics.strDot || qdir[i] == statics.strDotDot)
                     continue;
                 QString fname = dir + qdir[i];
@@ -1052,7 +1033,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
                     if (recursive)
                         dirs.append(fname + QLatin1Char('/'));
                 }
-                if (regex.exactMatch(qdir[i]))
+                if (regex.match(qdir[i]).hasMatch())
                       ret += ProString(fname).setSource(currentFileId());
             }
         }
@@ -1081,13 +1062,17 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
                 evalError(fL1S("Unexpected EOF."));
                 return ReturnError;
             }
-            ret = split_value_list(QStringRef(&line));
+            ret = split_value_list(QStringView(line));
         }
         break;
     }
 #endif
     case E_REPLACE: {
-        const QRegExp before(args.at(1).toQString());
+        const QRegularExpression before(args.at(1).toQString(), QRegularExpression::DotMatchesEverythingOption);
+        if (!before.isValid()) {
+            evalError(fL1S("replace(): Encountered invalid regular expression '%1'.").arg(before.pattern()));
+            goto allfail;
+        }
         ProStringRwUser u2(args.at(2), m_tmp2);
         const QString &after = u2.str();
         const auto vals = values(map(args.at(0)));
@@ -1110,7 +1095,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinExpand(
         ProString priosfx = args.count() < 4 ? ProString(".priority") : args.at(3);
         populateDeps(orgList, prefix,
                      args.count() < 3 ? ProStringList(ProString(".depends"))
-                                      : split_value_list(args.at(2).toQStringRef()),
+                                      : split_value_list(args.at(2).toQStringView()),
                      priosfx, dependencies, dependees, rootSet);
         while (!rootSet.isEmpty()) {
             QMultiMap<int, ProString>::iterator it = rootSet.begin();
@@ -1268,7 +1253,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::testFunc_cache(const ProStringList &
     enum { CacheSet, CacheAdd, CacheSub } mode = CacheSet;
     ProKey srcvar;
     if (args.count() >= 2) {
-        const auto opts = split_value_list(args.at(1).toQStringRef());
+        const auto opts = split_value_list(args.at(1).toQStringView());
         for (const ProString &opt : opts) {
             if (opt == QLatin1String("transient")) {
                 persist = false;
@@ -1458,15 +1443,15 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
     }
     case T_EXPORT: {
         const ProKey &var = map(args.at(0));
-        for (ProValueMapStack::Iterator vmi = m_valuemapStack.end();
+        for (ProValueMapStack::iterator vmi = m_valuemapStack.end();
              --vmi != m_valuemapStack.begin(); ) {
             ProValueMap::Iterator it = (*vmi).find(var);
             if (it != (*vmi).end()) {
                 if (it->constBegin() == statics.fakeValue.constBegin()) {
                     // This is stupid, but qmake doesn't propagate deletions
-                    m_valuemapStack.first()[var] = ProStringList();
+                    m_valuemapStack.front()[var] = ProStringList();
                 } else {
-                    m_valuemapStack.first()[var] = *it;
+                    m_valuemapStack.front()[var] = *it;
                 }
                 (*vmi).erase(it);
                 while (--vmi != m_valuemapStack.begin())
@@ -1477,7 +1462,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         return ReturnTrue;
     }
     case T_DISCARD_FROM: {
-        if (m_valuemapStack.count() != 1) {
+        if (m_valuemapStack.size() != 1) {
             evalError(fL1S("discard_from() cannot be called from functions."));
             return ReturnFalse;
         }
@@ -1487,13 +1472,13 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         int pro = m_vfs->idForFileName(fn, flags | QMakeVfs::VfsAccessedOnly);
         if (!pro)
             return ReturnFalse;
-        ProValueMap &vmap = m_valuemapStack.first();
+        ProValueMap &vmap = m_valuemapStack.front();
         for (auto vit = vmap.begin(); vit != vmap.end(); ) {
             if (!vit->isEmpty()) {
                 auto isFrom = [pro](const ProString &s) {
                     return s.sourceFile() == pro;
                 };
-                vit->erase(std::remove_if(vit->begin(), vit->end(), isFrom), vit->end());
+                vit->removeIf(isFrom);
                 if (vit->isEmpty()) {
                     // When an initially non-empty variable becomes entirely empty,
                     // undefine it altogether.
@@ -1515,7 +1500,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             else
                 ++fit;
         }
-        ProStringList &iif = m_valuemapStack.first()[ProKey("QMAKE_INTERNAL_INCLUDED_FILES")];
+        ProStringList &iif = m_valuemapStack.front()[ProKey("QMAKE_INTERNAL_INCLUDED_FILES")];
         int idx = iif.indexOf(ProString(fn));
         if (idx >= 0)
             iif.removeAt(idx);
@@ -1529,21 +1514,24 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             return ok;
         if (args.count() == 2)
             return returnBool(vars.contains(map(args.at(1))));
-        QRegExp regx;
+        QRegularExpression regx;
+        regx.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
         ProStringRoUser u1(args.at(2), m_tmp1);
         const QString &qry = u1.str();
-        if (qry != QRegExp::escape(qry)) {
-            QString copy = qry;
-            copy.detach();
-            regx.setPattern(copy);
+        if (qry != QRegularExpression::escape(qry)) {
+            regx.setPattern(QRegularExpression::anchoredPattern(qry));
+            if (!regx.isValid()) {
+                evalError(fL1S("infile(): Encountered invalid regular expression '%1'.").arg(qry));
+                return ReturnFalse;
+            }
         }
         const auto strings = vars.value(map(args.at(1)));
         for (const ProString &s : strings) {
             if (s == qry)
                 return ReturnTrue;
-            if (!regx.isEmpty()) {
+            if (!regx.pattern().isEmpty()) {
                 ProStringRoUser u2(s, m_tmp[m_toggle ^= 1]);
-                if (regx.exactMatch(u2.str()))
+                if (regx.match(u2.str()).hasMatch())
                     return ReturnTrue;
             }
         }
@@ -1558,7 +1546,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
     case T_EVAL: {
         VisitReturn ret = ReturnFalse;
         QString contents = args.join(statics.field_sep);
-        ProFile *pro = m_parser->parsedProBlock(QStringRef(&contents),
+        ProFile *pro = m_parser->parsedProBlock(QStringView(contents),
                                                 0, m_current.pro->fileName(), m_current.line);
         if (m_cumulative || pro->isOk()) {
             m_locationStack.push(m_current);
@@ -1570,19 +1558,19 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
         return ret;
     }
     case T_IF: {
-        return evaluateConditional(args.at(0).toQStringRef(),
+        return evaluateConditional(args.at(0).toQStringView(),
                                    m_current.pro->fileName(), m_current.line);
     }
     case T_CONFIG: {
         if (args.count() == 1)
-            return returnBool(isActiveConfig(args.at(0).toQStringRef()));
-        const auto mutuals = args.at(1).toQStringRef().split(QLatin1Char('|'),
-                                                             QString::SkipEmptyParts);
+            return returnBool(isActiveConfig(args.at(0).toQStringView()));
+        const auto mutuals = args.at(1).toQStringView().split(QLatin1Char('|'),
+                                                             Qt::SkipEmptyParts);
         const ProStringList &configs = values(statics.strCONFIG);
 
         for (int i = configs.size() - 1; i >= 0; i--) {
             for (int mut = 0; mut < mutuals.count(); mut++) {
-                if (configs[i].toQStringRef() == mutuals[mut].trimmed())
+                if (configs[i].toQStringView() == mutuals[mut].trimmed())
                     return returnBool(configs[i] == args[0]);
             }
         }
@@ -1591,11 +1579,14 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
     case T_CONTAINS: {
         ProStringRoUser u1(args.at(1), m_tmp1);
         const QString &qry = u1.str();
-        QRegExp regx;
-        if (qry != QRegExp::escape(qry)) {
-            QString copy = qry;
-            copy.detach();
-            regx.setPattern(copy);
+        QRegularExpression regx;
+        regx.setPatternOptions(QRegularExpression::DotMatchesEverythingOption);
+        if (qry != QRegularExpression::escape(qry)) {
+            regx.setPattern(QRegularExpression::anchoredPattern(qry));
+            if (!regx.isValid()) {
+                evalError(fL1S("contains(): Encountered invalid regular expression '%1'.").arg(qry));
+                return ReturnFalse;
+            }
         }
         const ProStringList &l = values(map(args.at(0)));
         if (args.count() == 2) {
@@ -1603,24 +1594,24 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
                 const ProString &val = l[i];
                 if (val == qry)
                     return ReturnTrue;
-                if (!regx.isEmpty()) {
+                if (!regx.pattern().isEmpty()) {
                     ProStringRoUser u2(val, m_tmp[m_toggle ^= 1]);
-                    if (regx.exactMatch(u2.str()))
+                    if (regx.match(u2.str()).hasMatch())
                         return ReturnTrue;
                 }
             }
         } else {
-            const auto mutuals = args.at(2).toQStringRef().split(QLatin1Char('|'),
-                                                                 QString::SkipEmptyParts);
+            const auto mutuals = args.at(2).toQStringView().split(QLatin1Char('|'),
+                                                                 Qt::SkipEmptyParts);
             for (int i = l.size() - 1; i >= 0; i--) {
                 const ProString &val = l[i];
                 for (int mut = 0; mut < mutuals.count(); mut++) {
-                    if (val.toQStringRef() == mutuals[mut].trimmed()) {
+                    if (val.toQStringView() == mutuals[mut].trimmed()) {
                         if (val == qry)
                             return ReturnTrue;
-                        if (!regx.isEmpty()) {
+                        if (!regx.pattern().isEmpty()) {
                             ProStringRoUser u2(val, m_tmp[m_toggle ^= 1]);
-                            if (regx.exactMatch(u2.str()))
+                            if (regx.match(u2.str()).hasMatch())
                                 return ReturnTrue;
                         }
                         return ReturnFalse;
@@ -1668,15 +1659,15 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             }
         }
         if (func_t == T_GREATERTHAN)
-            return returnBool(lhs > rhs.toQStringRef());
-        return returnBool(lhs < rhs.toQStringRef());
+            return returnBool(lhs > rhs.toQStringView());
+        return returnBool(lhs < rhs.toQStringView());
     }
     case T_EQUALS:
         return returnBool(values(map(args.at(0))).join(statics.field_sep)
                           == args.at(1).toQStringView());
     case T_VERSION_AT_LEAST:
     case T_VERSION_AT_MOST: {
-        const QVersionNumber lvn = QVersionNumber::fromString(values(args.at(0).toKey()).join('.'));
+        const QVersionNumber lvn = QVersionNumber::fromString(values(args.at(0).toKey()).join(QLatin1Char('.')));
         const QVersionNumber rvn = QVersionNumber::fromString(args.at(1).toQStringView());
         if (func_t == T_VERSION_AT_LEAST)
             return returnBool(lvn >= rvn);
@@ -1708,14 +1699,12 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             m_valuemapStack.top()[var] = statics.fakeValue;
         return ReturnTrue;
     }
-#if QT_VERSION >= QT_VERSION_CHECK(5, 0, 0)
     case T_PARSE_JSON: {
         QByteArray json = values(args.at(0).toKey()).join(QLatin1Char(' ')).toUtf8();
         ProStringRoUser u1(args.at(1), m_tmp2);
         QString parseInto = u1.str();
         return parseJsonInto(json, parseInto, &m_valuemapStack.top());
     }
-#endif
     case T_INCLUDE: {
         QString parseInto;
         LoadFlags flags;
@@ -1860,7 +1849,7 @@ QMakeEvaluator::VisitReturn QMakeEvaluator::evaluateBuiltinConditional(
             if (!vals.isEmpty())
                 contents = vals.join(QLatin1Char('\n')) + QLatin1Char('\n');
             if (args.count() >= 3) {
-                const auto opts = split_value_list(args.at(2).toQStringRef());
+                const auto opts = split_value_list(args.at(2).toQStringView());
                 for (const ProString &opt : opts) {
                     if (opt == QLatin1String("append")) {
                         mode = QIODevice::Append;
