@@ -1,31 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2018 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 or (at your option) any later version
-** approved by the KDE Free Qt Foundation. The licenses are as published by
-** the Free Software Foundation and appearing in the file LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2018 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only
 
 #include <qpa/qwindowsysteminterface.h>
 #include <private/qguiapplication_p.h>
@@ -40,10 +14,11 @@
 
 #include <iostream>
 
-Q_GUI_EXPORT int qt_defaultDpiX();
 
 QT_BEGIN_NAMESPACE
 
+Q_GUI_EXPORT int qt_defaultDpiX();
+
 QWasmWindow::QWasmWindow(QWindow *w, QWasmCompositor *compositor, QWasmBackingStore *backingStore)
     : QPlatformWindow(w),
       m_window(w),
@@ -109,9 +84,18 @@ void QWasmWindow::setGeometry(const QRect &rect)
         if (r.y() < yMin)
             r.moveTop(yMin);
     }
+    bool shouldInvalidate = true;
+    if (!m_windowState.testFlag(Qt::WindowFullScreen)
+        && !m_windowState.testFlag(Qt::WindowMaximized)) {
+        shouldInvalidate = m_normalGeometry.size() != r.size();
+        m_normalGeometry = r;
+    }
     QPlatformWindow::setGeometry(r);
     QWindowSystemInterface::handleGeometryChange(window(), r);
-    invalidate();
+    if (shouldInvalidate)
+        invalidate();
+    else
+        m_compositor->requestUpdate();
 }
 
 void QWasmWindow::setVisible(bool visible)
@@ -143,14 +127,16 @@ QMargins QWasmWindow::frameMargins() const
 void QWasmWindow::raise()
 {
     m_compositor->raise(this);
-    QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(0, 0), geometry().size()));
+    if (window()->isVisible())
+        QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(0, 0), geometry().size()));
     invalidate();
 }
 
 void QWasmWindow::lower()
 {
     m_compositor->lower(this);
-    QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(0, 0), geometry().size()));
+    if (window()->isVisible())
+        QWindowSystemInterface::handleExposeEvent(window(), QRect(QPoint(0, 0), geometry().size()));
     invalidate();
 }
 
@@ -252,15 +238,13 @@ QRegion QWasmWindow::resizeRegion() const
 
 bool QWasmWindow::isPointOnTitle(QPoint point) const
 {
-    bool ok = titleGeometry().contains(point);
-    return ok;
+    return hasTitleBar() ? titleGeometry().contains(point) : false;
 }
 
 bool QWasmWindow::isPointOnResizeRegion(QPoint point) const
 {
-    if (window()->flags().testFlag(Qt::Popup))
-        return false;
-    return resizeRegion().contains(point);
+    return (window()->maximumSize().isEmpty() || window()->minimumSize() != window()->maximumSize())
+            && resizeRegion().contains(point);
 }
 
 QWasmCompositor::ResizeMode QWasmWindow::resizeModeAtPoint(QPoint point) const
@@ -456,4 +440,13 @@ void QWasmWindow::requestActivateWindow()
     QPlatformWindow::requestActivateWindow();
 }
 
+bool QWasmWindow::setMouseGrabEnabled(bool grab)
+{
+    if (grab)
+        m_compositor->setCapture(this);
+    else
+        m_compositor->releaseCapture();
+    return true;
+}
+
 QT_END_NAMESPACE
