@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the plugins of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 #include <QDebug>
 
@@ -51,18 +15,19 @@
 #if QT_CONFIG(regularexpression)
 #  include <QtCore/QRegularExpression>
 #endif
+#include <QtGui/qguiapplication.h>
 #include <QtGui/QOpenGLContext>
 #include <QtGui/QOffscreenSurface>
 
 #include "qglxintegration.h"
-#include <QtGlxSupport/private/qglxconvenience_p.h>
-#include <QtPlatformHeaders/QGLXNativeContext>
+#include <QtGui/private/qglxconvenience_p.h>
 
 #include "qxcbglintegration.h"
 
 QT_BEGIN_NAMESPACE
 
 typedef GLXContext (*glXCreateContextAttribsARBProc)(Display*, GLXFBConfig, GLXContext, Bool, const int*);
+typedef const GLubyte *(*glGetStringiProc)(GLenum, GLuint);
 
 #ifndef GLX_CONTEXT_CORE_PROFILE_BIT_ARB
 #define GLX_CONTEXT_CORE_PROFILE_BIT_ARB 0x00000001
@@ -145,6 +110,27 @@ static inline QByteArray getGlString(GLenum param)
     return QByteArray();
 }
 
+static bool hasGlExtension(const QSurfaceFormat &format, const char *ext)
+{
+    if (format.majorVersion() < 3) {
+        auto exts = reinterpret_cast<const char *>(glGetString(GL_EXTENSIONS));
+        return exts && strstr(exts, ext);
+    } else {
+        auto glGetStringi = reinterpret_cast<glGetStringiProc>(
+                glXGetProcAddress(reinterpret_cast<const GLubyte*>("glGetStringi")));
+        if (glGetStringi) {
+            GLint n = 0;
+            glGetIntegerv(GL_NUM_EXTENSIONS, &n);
+            for (GLint i = 0; i < n; ++i) {
+                const char *p = reinterpret_cast<const char *>(glGetStringi(GL_EXTENSIONS, i));
+                if (p && !strcmp(p, ext))
+                    return true;
+            }
+        }
+        return false;
+    }
+}
+
 static void updateFormatFromContext(QSurfaceFormat &format)
 {
     // Update the version, profile, and context bit of the format
@@ -163,7 +149,7 @@ static void updateFormatFromContext(QSurfaceFormat &format)
         format.setOption(QSurfaceFormat::StereoBuffers);
 
     if (format.renderableType() == QSurfaceFormat::OpenGL) {
-        if (format.version() >= qMakePair(4, 0)) {
+        if (hasGlExtension(format, "GL_ARB_robustness")) {
             GLint value = 0;
             glGetIntegerv(GL_RESET_NOTIFICATION_STRATEGY_ARB, &value);
             if (value == GL_LOSE_CONTEXT_ON_RESET_ARB)
@@ -197,33 +183,15 @@ static void updateFormatFromContext(QSurfaceFormat &format)
     }
 }
 
-QGLXContext::QGLXContext(QXcbScreen *screen, const QSurfaceFormat &format, QPlatformOpenGLContext *share,
-                         const QVariant &nativeHandle)
+QGLXContext::QGLXContext(Display *display, QXcbScreen *screen, const QSurfaceFormat &format, QPlatformOpenGLContext *share)
     : QPlatformOpenGLContext()
-    , m_display(static_cast<Display *>(screen->connection()->xlib_display()))
-    , m_config(0)
-    , m_context(0)
-    , m_shareContext(0)
+    , m_display(display)
     , m_format(format)
-    , m_isPBufferCurrent(false)
-    , m_ownsContext(nativeHandle.isNull())
-    , m_getGraphicsResetStatus(0)
-    , m_lost(false)
-{
-    if (nativeHandle.isNull())
-        init(screen, share);
-    else
-        init(screen, share, nativeHandle);
-}
-
-void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
+    , m_ownsContext(true)
 {
     if (m_format.renderableType() == QSurfaceFormat::DefaultRenderableType)
-#if defined(QT_OPENGL_ES_2)
-        m_format.setRenderableType(QSurfaceFormat::OpenGLES);
-#else
-        m_format.setRenderableType(QSurfaceFormat::OpenGL);
-#endif
+        m_format.setRenderableType(QOpenGLContext::openGLModuleType() == QOpenGLContext::LibGL
+                                   ? QSurfaceFormat::OpenGL : QSurfaceFormat::OpenGLES);
     if (m_format.renderableType() != QSurfaceFormat::OpenGL && m_format.renderableType() != QSurfaceFormat::OpenGLES)
         return;
 
@@ -232,14 +200,14 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
 
     GLXFBConfig config = qglx_findConfig(m_display, screen->screenNumber(), m_format);
     m_config = config;
-    XVisualInfo *visualInfo = 0;
+    XVisualInfo *visualInfo = nullptr;
     Window window = 0; // Temporary window used to query OpenGL context
 
     if (config) {
         const QByteArrayList glxExt = QByteArray(glXQueryExtensionsString(m_display, screen->screenNumber())).split(' ');
 
         // Resolve entry point for glXCreateContextAttribsARB
-        glXCreateContextAttribsARBProc glXCreateContextAttribsARB = 0;
+        glXCreateContextAttribsARBProc glXCreateContextAttribsARB = nullptr;
         if (glxExt.contains("GLX_ARB_create_context"))
             glXCreateContextAttribsARB = (glXCreateContextAttribsARBProc) glXGetProcAddress((const GLubyte*)"glXCreateContextAttribsARB");
 
@@ -249,13 +217,13 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
 
         // Use glXCreateContextAttribsARB if available
         // Also, GL ES context creation requires GLX_EXT_create_context_es2_profile
-        if (glXCreateContextAttribsARB != 0
+        if (glXCreateContextAttribsARB != nullptr
                 && (m_format.renderableType() != QSurfaceFormat::OpenGLES || (supportsProfiles && glxExt.contains("GLX_EXT_create_context_es2_profile")))) {
             // Try to create an OpenGL context for each known OpenGL version in descending
             // order from the requested version.
             const int requestedVersion = m_format.majorVersion() * 10 + qMin(m_format.minorVersion(), 9);
 
-            QVector<int> glVersions;
+            QList<int> glVersions;
             if (m_format.renderableType() == QSurfaceFormat::OpenGL) {
                 if (requestedVersion > 46)
                     glVersions << requestedVersion;
@@ -284,7 +252,7 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
                 const int majorVersion = version / 10;
                 const int minorVersion = version % 10;
 
-                QVector<int> contextAttributes;
+                QList<int> contextAttributes;
                 contextAttributes << GLX_CONTEXT_MAJOR_VERSION_ARB << majorVersion
                                   << GLX_CONTEXT_MINOR_VERSION_ARB << minorVersion;
 
@@ -317,7 +285,7 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
                 }
 
                 if (supportsRobustness && m_format.testOption(QSurfaceFormat::ResetNotification)) {
-                    QVector<int> contextAttributesWithRobustness = contextAttributes;
+                    QList<int> contextAttributesWithRobustness = contextAttributes;
                     contextAttributesWithRobustness << GLX_CONTEXT_RESET_NOTIFICATION_STRATEGY_ARB << GLX_LOSE_CONTEXT_ON_RESET_ARB;
                     if (supportsVideoMemoryPurge)
                         contextAttributesWithRobustness << GLX_GENERATE_RESET_ON_VIDEO_MEMORY_PURGE_NV << GL_TRUE;
@@ -336,9 +304,9 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
                     m_context = glXCreateContextAttribsARB(m_display, config, m_shareContext, true, contextAttributes.data());
                     if (!m_context && m_shareContext) {
                         // re-try without a shared glx context
-                        m_context = glXCreateContextAttribsARB(m_display, config, 0, true, contextAttributes.data());
+                        m_context = glXCreateContextAttribsARB(m_display, config, nullptr, true, contextAttributes.data());
                         if (m_context)
-                            m_shareContext = 0;
+                            m_shareContext = nullptr;
                     }
                 }
             }
@@ -353,9 +321,9 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
             m_context = glXCreateNewContext(m_display, config, GLX_RGBA_TYPE, m_shareContext, true);
             if (!m_context && m_shareContext) {
                 // re-try without a shared glx context
-                m_context = glXCreateNewContext(m_display, config, GLX_RGBA_TYPE, 0, true);
+                m_context = glXCreateNewContext(m_display, config, GLX_RGBA_TYPE, nullptr, true);
                 if (m_context)
-                    m_shareContext = 0;
+                    m_shareContext = nullptr;
             }
         }
 
@@ -377,7 +345,7 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
         m_context = glXCreateContext(m_display, visualInfo, m_shareContext, true);
         if (!m_context && m_shareContext) {
             // re-try without a shared glx context
-            m_shareContext = 0;
+            m_shareContext = nullptr;
             m_context = glXCreateContext(m_display, visualInfo, nullptr, true);
         }
 
@@ -401,65 +369,32 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share)
     XDestroyWindow(m_display, window);
 }
 
-void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share, const QVariant &nativeHandle)
+QGLXContext::QGLXContext(Display *display, GLXContext context, void *visualInfo, QPlatformOpenGLContext *share)
+    : QPlatformOpenGLContext()
+    , m_display(display)
 {
-    if (!nativeHandle.canConvert<QGLXNativeContext>()) {
-        qWarning("QGLXContext: Requires a QGLXNativeContext");
-        return;
-    }
-    QGLXNativeContext handle = nativeHandle.value<QGLXNativeContext>();
-    GLXContext context = handle.context();
-    if (!context) {
-        qWarning("QGLXContext: No GLXContext given");
-        return;
-    }
+    // Legacy contexts created using glXCreateContext are created using a
+    // XVisualInfo. If the user passed one we should use that.
+    XVisualInfo *vinfo = static_cast<XVisualInfo*>(visualInfo);
 
-    // Use the provided Display, if available. If not, use our own. It may still work.
-    Display *dpy = handle.display();
-    if (!dpy)
-        dpy = m_display;
-
-    // Legacy contexts created using glXCreateContext are created using a visual
-    // and the FBConfig cannot be queried. The only way to adapt these contexts
-    // is to figure out the visual id.
-    XVisualInfo *vinfo = 0;
-    // If the VisualID is provided use it.
-    VisualID vid = handle.visualId();
-    if (!vid) {
-        // In the absence of the VisualID figure it out from the window.
-        Window wnd = handle.window();
-        if (wnd) {
-            XWindowAttributes attrs;
-            XGetWindowAttributes(dpy, wnd, &attrs);
-            vid = XVisualIDFromVisual(attrs.visual);
-        }
-    }
-    if (vid) {
-        XVisualInfo v;
-        v.screen = screen->screenNumber();
-        v.visualid = vid;
-        int n = 0;
-        vinfo = XGetVisualInfo(dpy, VisualScreenMask | VisualIDMask, &v, &n);
-        if (n < 1) {
-            XFree(vinfo);
-            vinfo = 0;
-        }
-    }
-
-    // For contexts created with an FBConfig using the modern functions providing the
-    // visual or window is not mandatory. Just query the config from the context.
-    GLXFBConfig config = 0;
+    // Otherwise assume the context was created with an FBConfig using the modern functions
     if (!vinfo) {
         int configId = 0;
-        if (glXQueryContext(dpy, context, GLX_FBCONFIG_ID, &configId) != Success) {
+        if (glXQueryContext(m_display, context, GLX_FBCONFIG_ID, &configId) != Success) {
             qWarning("QGLXContext: Failed to query config from the provided context");
             return;
         }
 
+        int screenNumber = 0;
+        if (glXQueryContext(m_display, context, GLX_SCREEN, &screenNumber) != Success) {
+            qWarning("QGLXContext: Failed to query screen from the provided context");
+            screenNumber = DefaultScreen(m_display);
+        }
+
         GLXFBConfig *configs;
         int numConfigs = 0;
         static const int attribs[] = { GLX_FBCONFIG_ID, configId, None };
-        configs = glXChooseFBConfig(dpy, screen->screenNumber(), attribs, &numConfigs);
+        configs = glXChooseFBConfig(m_display, screenNumber, attribs, &numConfigs);
         if (!configs || numConfigs < 1) {
             qWarning("QGLXContext: Failed to find config");
             return;
@@ -467,19 +402,17 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share, const
         if (configs && numConfigs > 1) // this is suspicious so warn but let it continue
             qWarning("QGLXContext: Multiple configs for FBConfig ID %d", configId);
 
-        config = configs[0];
-        // Store the config.
-        m_config = config;
+        m_config = configs[0];
     }
 
-    Q_ASSERT(vinfo || config);
+    Q_ASSERT(vinfo || m_config);
 
-    int screenNumber = DefaultScreen(dpy);
+    int screenNumber = DefaultScreen(m_display);
     Window window;
     if (vinfo)
-        window = createDummyWindow(dpy, vinfo, screenNumber, RootWindow(dpy, screenNumber));
+        window = createDummyWindow(m_display, vinfo, screenNumber, RootWindow(m_display, screenNumber));
     else
-        window = createDummyWindow(dpy, config, screenNumber, RootWindow(dpy, screenNumber));
+        window = createDummyWindow(m_display, m_config, screenNumber, RootWindow(m_display, screenNumber));
     if (!window) {
         qWarning("QGLXContext: Failed to create dummy window");
         return;
@@ -488,7 +421,7 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share, const
     // Update OpenGL version and buffer sizes in our format.
     GLXContext prevContext = glXGetCurrentContext();
     GLXDrawable prevDrawable = glXGetCurrentDrawable();
-    if (!glXMakeCurrent(dpy, window, context)) {
+    if (!glXMakeCurrent(m_display, window, context)) {
         qWarning("QGLXContext: Failed to make provided context current");
         return;
     }
@@ -497,11 +430,11 @@ void QGLXContext::init(QXcbScreen *screen, QPlatformOpenGLContext *share, const
                                ? QSurfaceFormat::OpenGL : QSurfaceFormat::OpenGLES);
     updateFormatFromContext(m_format);
     if (vinfo)
-        qglx_surfaceFormatFromVisualInfo(&m_format, dpy, vinfo);
+        qglx_surfaceFormatFromVisualInfo(&m_format, m_display, vinfo);
     else
-        qglx_surfaceFormatFromGLXFBConfig(&m_format, dpy, config);
-    glXMakeCurrent(dpy, prevDrawable, prevContext);
-    XDestroyWindow(dpy, window);
+        qglx_surfaceFormatFromGLXFBConfig(&m_format, m_display, m_config);
+    glXMakeCurrent(m_display, prevDrawable, prevContext);
+    XDestroyWindow(m_display, window);
 
     if (vinfo)
         XFree(vinfo);
@@ -530,11 +463,6 @@ static QXcbScreen *screenForPlatformSurface(QPlatformSurface *surface)
     return nullptr;
 }
 
-QVariant QGLXContext::nativeHandle() const
-{
-    return QVariant::fromValue<QGLXNativeContext>(QGLXNativeContext(m_context));
-}
-
 bool QGLXContext::makeCurrent(QPlatformSurface *surface)
 {
     bool success = false;
@@ -573,8 +501,8 @@ bool QGLXContext::makeCurrent(QPlatformSurface *surface)
         if (interval >= 0 && interval != window->swapInterval() && screen) {
             typedef void (*qt_glXSwapIntervalEXT)(Display *, GLXDrawable, int);
             typedef void (*qt_glXSwapIntervalMESA)(unsigned int);
-            static qt_glXSwapIntervalEXT glXSwapIntervalEXT = 0;
-            static qt_glXSwapIntervalMESA glXSwapIntervalMESA = 0;
+            static qt_glXSwapIntervalEXT glXSwapIntervalEXT = nullptr;
+            static qt_glXSwapIntervalMESA glXSwapIntervalMESA = nullptr;
             static bool resolved = false;
             if (!resolved) {
                 resolved = true;
@@ -599,9 +527,9 @@ bool QGLXContext::makeCurrent(QPlatformSurface *surface)
 void QGLXContext::doneCurrent()
 {
     if (m_isPBufferCurrent)
-        glXMakeContextCurrent(m_display, 0, 0, 0);
+        glXMakeContextCurrent(m_display, 0, 0, nullptr);
     else
-        glXMakeCurrent(m_display, 0, 0);
+        glXMakeCurrent(m_display, 0, nullptr);
     m_isPBufferCurrent = false;
 }
 
@@ -636,12 +564,12 @@ QSurfaceFormat QGLXContext::format() const
 
 bool QGLXContext::isSharing() const
 {
-    return m_shareContext != 0;
+    return m_shareContext != nullptr;
 }
 
 bool QGLXContext::isValid() const
 {
-    return m_context != 0 && !m_lost;
+    return m_context != nullptr && !m_lost;
 }
 
 bool QGLXContext::m_queriedDummyContext = false;
@@ -653,7 +581,7 @@ bool QGLXContext::m_supportsThreading = true;
 // binary search.
 static const char *qglx_threadedgl_blacklist_renderer[] = {
     "Chromium",                             // QTBUG-32225 (initialization fails)
-    0
+    nullptr
 };
 
 static const char *qglx_threadedgl_blacklist_vendor[] = {
@@ -673,7 +601,7 @@ void QGLXContext::queryDummyContext()
         return;
 
     QOpenGLContext *oldContext = QOpenGLContext::currentContext();
-    QSurface *oldSurface = 0;
+    QSurface *oldSurface = nullptr;
     if (oldContext)
         oldSurface = oldContext->surface();
 
@@ -710,7 +638,7 @@ void QGLXContext::queryDummyContext()
 
     if (const char *renderer = (const char *) glGetString(GL_RENDERER)) {
         for (int i = 0; qglx_threadedgl_blacklist_renderer[i]; ++i) {
-            if (strstr(renderer, qglx_threadedgl_blacklist_renderer[i]) != 0) {
+            if (strstr(renderer, qglx_threadedgl_blacklist_renderer[i]) != nullptr) {
                 qCDebug(lcQpaGl).nospace() << "Multithreaded OpenGL disabled: "
                                              "blacklisted renderer \""
                                           << qglx_threadedgl_blacklist_renderer[i]
@@ -722,7 +650,7 @@ void QGLXContext::queryDummyContext()
     }
     if (const char *vendor = (const char *) glGetString(GL_VENDOR)) {
         for (int i = 0; qglx_threadedgl_blacklist_vendor[i]; ++i) {
-            if (strstr(vendor, qglx_threadedgl_blacklist_vendor[i]) != 0) {
+            if (strstr(vendor, qglx_threadedgl_blacklist_vendor[i]) != nullptr) {
                 qCDebug(lcQpaGl).nospace() << "Multithreaded OpenGL disabled: "
                                               "blacklisted vendor \""
                                            << qglx_threadedgl_blacklist_vendor[i]
@@ -737,7 +665,7 @@ void QGLXContext::queryDummyContext()
         // Blacklist Mesa drivers due to QTCREATORBUG-10875 (crash in creator),
         // QTBUG-34492 (flickering in fullscreen) and QTBUG-38221
         const char *mesaVersionStr = nullptr;
-        if (strstr(glxvendor, "Mesa Project") != 0) {
+        if (strstr(glxvendor, "Mesa Project") != nullptr) {
             mesaVersionStr = (const char *) glGetString(GL_VERSION);
             m_supportsThreading = false;
         }
@@ -764,6 +692,10 @@ void QGLXContext::queryDummyContext()
         }
     }
 
+    static bool nomultithread = qEnvironmentVariableIsSet("QT_XCB_NO_THREADED_OPENGL");
+    if (nomultithread)
+        m_supportsThreading = false;
+
     context.doneCurrent();
     if (oldContext && oldSurface)
         oldContext->makeCurrent(oldSurface);
