@@ -1,41 +1,5 @@
-/****************************************************************************
-**
-** Copyright (C) 2016 The Qt Company Ltd.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the QtWidgets module of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:LGPL$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU Lesser General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 3 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL3 included in the
-** packaging of this file. Please review the following information to
-** ensure the GNU Lesser General Public License version 3 requirements
-** will be met: https://www.gnu.org/licenses/lgpl-3.0.html.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 2.0 or (at your option) the GNU General
-** Public license version 3 or any later version approved by the KDE Free
-** Qt Foundation. The licenses are as published by the Free Software
-** Foundation and appearing in the file LICENSE.GPL2 and LICENSE.GPL3
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-2.0.html and
-** https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2016 The Qt Company Ltd.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR LGPL-3.0-only OR GPL-2.0-only OR GPL-3.0-only
 
 /*!
     \class QMdiArea
@@ -176,6 +140,7 @@
 
 QT_BEGIN_NAMESPACE
 
+using namespace Qt::StringLiterals;
 using namespace QMdi;
 
 // Asserts in debug mode, gives warning otherwise.
@@ -277,7 +242,7 @@ static inline QString tabTextFor(QMdiSubWindow *subWindow)
 
     QString title = subWindow->windowTitle();
     if (subWindow->isWindowModified()) {
-        title.replace(QLatin1String("[*]"), QLatin1String("*"));
+        title.replace("[*]"_L1, "*"_L1);
     } else {
         extern QString qt_setWindowTitle_helperHelper(const QString&, const QWidget*);
         title = qt_setWindowTitle_helperHelper(title, subWindow);
@@ -1000,6 +965,10 @@ void QMdiAreaPrivate::activateWindow(QMdiSubWindow *child)
 
     if (child->isHidden() || child == active)
         return;
+
+    if (child->d_func()->isActive && active == nullptr)
+        child->d_func()->isActive = false;
+
     child->d_func()->setActive(true);
 }
 
@@ -1511,7 +1480,7 @@ void QMdiAreaPrivate::highlightNextSubWindow(int increaseFactor)
     if (!rubberBand) {
         rubberBand = new QRubberBand(QRubberBand::Rectangle, q);
         // For accessibility to identify this special widget.
-        rubberBand->setObjectName(QLatin1String("qt_rubberband"));
+        rubberBand->setObjectName("qt_rubberband"_L1);
         rubberBand->setWindowFlags(rubberBand->windowFlags() | Qt::WindowStaysOnTopHint);
     }
 #endif
@@ -2314,7 +2283,14 @@ void QMdiArea::resizeEvent(QResizeEvent *resizeEvent)
     foreach (QMdiSubWindow *child, d->childWindows) {
         if (sanityCheck(child, "QMdiArea::resizeEvent") && child->isMaximized()
                 && child->size() != resizeEvent->size()) {
-            child->resize(resizeEvent->size());
+            auto realSize = resizeEvent->size();
+            const auto minSizeHint = child->minimumSizeHint();
+            // QMdiSubWindow is no tlw so minimumSize() is not set by the layout manager
+            // and therefore we have to take care by ourself that we're not getting smaller
+            // than allowed
+            if (minSizeHint.isValid())
+                realSize = realSize.expandedTo(minSizeHint);
+            child->resize(realSize);
             if (!hasMaximizedSubWindow)
                 hasMaximizedSubWindow = true;
         }
@@ -2547,7 +2523,7 @@ bool QMdiArea::eventFilter(QObject *object, QEvent *event)
     if (event->type() == QEvent::KeyPress || event->type() == QEvent::KeyRelease) {
 
         QKeyEvent *keyEvent = static_cast<QKeyEvent *>(event);
-        // Ingore key events without a Ctrl modifier (except for press/release on the modifier itself).
+        // Ignore key events without a Ctrl modifier (except for press/release on the modifier itself).
         if (!(keyEvent->modifiers() & Qt::ControlModifier) && keyEvent->key() != Qt::Key_Control)
             return QAbstractScrollArea::eventFilter(object, event);
 
