@@ -1,31 +1,6 @@
-/****************************************************************************
-**
-** Copyright (C) 2021 The Qt Company Ltd.
-** Copyright (C) 2016 Intel Corporation.
-** Contact: https://www.qt.io/licensing/
-**
-** This file is part of the test suite of the Qt Toolkit.
-**
-** $QT_BEGIN_LICENSE:GPL-EXCEPT$
-** Commercial License Usage
-** Licensees holding valid commercial Qt licenses may use this file in
-** accordance with the commercial license agreement provided with the
-** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and The Qt Company. For licensing terms
-** and conditions see https://www.qt.io/terms-conditions. For further
-** information use the contact form at https://www.qt.io/contact-us.
-**
-** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU
-** General Public License version 3 as published by the Free Software
-** Foundation with exceptions as appearing in the file LICENSE.GPL3-EXCEPT
-** included in the packaging of this file. Please review the following
-** information to ensure the GNU General Public License requirements will
-** be met: https://www.gnu.org/licenses/gpl-3.0.html.
-**
-** $QT_END_LICENSE$
-**
-****************************************************************************/
+// Copyright (C) 2021 The Qt Company Ltd.
+// Copyright (C) 2016 Intel Corporation.
+// SPDX-License-Identifier: LicenseRef-Qt-Commercial OR GPL-3.0-only WITH Qt-GPL-exception-1.0
 
 #include <QtCore/QCoreApplication>
 
@@ -50,6 +25,8 @@ QT_REQUIRE_CONFIG(process);
 
 #include <QtTest/private/qemulationdetector_p.h>
 
+using namespace Qt::StringLiterals;
+
 struct BenchmarkResult
 {
     qint64  total;
@@ -313,8 +290,7 @@ bool compareLine(const QString &logger, const QString &subdir,
         return compareBenchmarkResult(actualResult, expectedResult, errorMessage);
     }
 
-    if (actualLine.startsWith(QLatin1String("    <Duration msecs="))
-        || actualLine.startsWith(QLatin1String("<Duration msecs="))) {
+    if (actualLine.contains(QLatin1String("<Duration msecs="))) {
         static QRegularExpression durationRegExp("<Duration msecs=\"[\\d\\.]+\"/>");
         QRegularExpressionMatch match = durationRegExp.match(actualLine);
         if (match.hasMatch())
@@ -677,7 +653,7 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
         return true;
 #endif
 
-#if defined(QT_NO_EXCEPTIONS) || defined(Q_CC_INTEL) || defined(Q_OS_WIN)
+#if defined(QT_NO_EXCEPTIONS) || defined(Q_OS_WIN)
     // Disable this test on Windows or for Intel compiler, as the run-times
     // will popup dialogs with warnings that uncaught exceptions were thrown
     if (test == "exceptionthrow")
@@ -694,7 +670,7 @@ bool TestLogger::shouldIgnoreTest(const QString &test) const
 #if defined(__GNUC__) && (defined(__i386) || defined(__x86_64)) && defined(Q_OS_LINUX)
         // Check that it's actually available
         QProcess checkProcess;
-        QStringList args{u"--version"_qs};
+        QStringList args{u"--version"_s};
         checkProcess.start("valgrind", args);
         if (!checkProcess.waitForFinished(-1)) {
             WARN("Valgrind broken or not available. Not running benchlibcallgrind test!");
@@ -1219,6 +1195,58 @@ SCENARIO("Test output of the loggers is as expected")
     }
 }
 
+struct TestCase {
+    int expectedExitCode;
+    const char *cmdline;
+};
+
+SCENARIO("Exit code is as expected")
+{
+    // Listing of test command lines and expected exit codes
+    // NOTE: Use at least 2 spaces to separate arguments because some contain a space themselves.
+    const struct TestCase testCases[] = {
+    // 'pass' is a test with no data tags at all
+        { 0, "pass  testNumber1" },
+        { 1, "pass  unknownFunction" },
+        { 1, "pass  testNumber1:blah" },
+        { 1, "pass  testNumber1:blah:blue" },
+    // 'counting' is a test that has only local data tags
+        { 0, "counting  testPassPass" },
+        { 0, "counting  testPassPass:row 1" },
+        { 1, "counting  testPassPass:blah" },
+        { 1, "counting  testPassPass:blah:row 1" },
+        { 1, "counting  testPassPass:blah:blue" },
+    // 'globaldata' is a test with global and local data tags
+        { 0, "globaldata  testGlobal" },
+        { 0, "globaldata  testGlobal:global=true" },
+        { 0, "globaldata  testGlobal:local=true" },
+        { 0, "globaldata  testGlobal:global=true:local=true" },
+        { 1, "globaldata  testGlobal:local=true:global=true" },
+        { 1, "globaldata  testGlobal:global=true:blah" },
+        { 1, "globaldata  testGlobal:blah:local=true" },
+        { 1, "globaldata  testGlobal:blah:global=true" },
+        { 1, "globaldata  testGlobal:blah" },
+        { 1, "globaldata  testGlobal:blah:blue" },
+    // Passing multiple testcase:data on the command line
+        { 0, "globaldata  testGlobal:global=true  skipSingle:global=true:local=true" },
+        { 1, "globaldata  testGlobal:blah         skipSingle:global=true:local=true" },
+        { 1, "globaldata  testGlobal:global=true  skipSingle:blah" },
+        { 2, "globaldata  testGlobal:blah         skipSingle:blue" },
+    };
+
+    size_t n_testCases = sizeof(testCases) / sizeof(*testCases);
+    for (size_t i = 0; i < n_testCases; i++) {
+        GIVEN("The command line: " << testCases[i].cmdline) {
+            const QStringList cmdSplit = QString(testCases[i].cmdline)
+                    .split(QRegularExpression("  +"));    // at least 2 spaces
+            const QString test     = cmdSplit[0];
+            const QStringList args = cmdSplit.sliced(1);
+            auto runResult = runTestProcess(test, args);
+            REQUIRE(runResult.exitCode == testCases[i].expectedExitCode);
+        }
+    }
+}
+
 // ----------------------- Entrypoint -----------------------
 
 int main(int argc, char **argv)
